@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -302,7 +302,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
-	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
 	fstream->owner = FALSE;
 	fstream->fp = GMIME_STREAM_FILE (stream)->fp;
@@ -315,9 +315,9 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
  * g_mime_stream_file_new:
  * @fp: file pointer
  *
- * Creates a new GMimeStreamFile object around @fp.
+ * Creates a new #GMimeStreamFile object around @fp.
  *
- * Note: The created GMimeStreamFile object will own the FILE pointer
+ * Note: The created #GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
  * Returns: a stream using @fp.
@@ -331,7 +331,7 @@ g_mime_stream_file_new (FILE *fp)
 	if ((start = ftell (fp)) == -1)
 		start = 0;
 	
-	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, -1);
 	fstream->owner = TRUE;
 	fstream->fp = fp;
@@ -346,10 +346,10 @@ g_mime_stream_file_new (FILE *fp)
  * @start: start boundary
  * @end: end boundary
  *
- * Creates a new GMimeStreamFile object around @fp with bounds @start
+ * Creates a new #GMimeStreamFile object around @fp with bounds @start
  * and @end.
  *
- * Note: The created GMimeStreamFile object will own the FILE pointer
+ * Note: The created #GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
  * Returns: a stream using @fp with bounds @start and @end.
@@ -359,7 +359,7 @@ g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
-	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
 	fstream->owner = TRUE;
 	fstream->fp = fp;
