@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -32,7 +32,7 @@
 
 
 #ifndef HAVE_FSYNC
-int fsync (int fd) { return 0; }
+static int fsync (int fd) { return 0; }
 #endif
 
 
@@ -347,7 +347,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	GMimeStreamFs *fs;
 	
-	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
+	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
 	fs->fd = GMIME_STREAM_FS (stream)->fd;
 	fs->owner = FALSE;
@@ -361,9 +361,9 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
  * g_mime_stream_fs_new:
  * @fd: file descriptor
  *
- * Creates a new GMimeStreamFs object around @fd.
+ * Creates a new #GMimeStreamFs object around @fd.
  *
- * Returns a stream using @fd.
+ * Returns: a stream using @fd.
  **/
 GMimeStream *
 g_mime_stream_fs_new (int fd)
@@ -374,7 +374,7 @@ g_mime_stream_fs_new (int fd)
 	if ((start = lseek (fd, (off_t) 0, SEEK_CUR)) == -1)
 		start = 0;
 	
-	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
+	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fs), start, -1);
 	fs->owner = TRUE;
 	fs->eos = FALSE;
@@ -390,17 +390,17 @@ g_mime_stream_fs_new (int fd)
  * @start: start boundary
  * @end: end boundary
  *
- * Creates a new GMimeStreamFs object around @fd with bounds @start
+ * Creates a new #GMimeStreamFs object around @fd with bounds @start
  * and @end.
  *
- * Returns a stream using @fd with bounds @start and @end.
+ * Returns: a stream using @fd with bounds @start and @end.
  **/
 GMimeStream *
 g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end)
 {
 	GMimeStreamFs *fs;
 	
-	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
+	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
 	fs->owner = TRUE;
 	fs->eos = FALSE;
@@ -416,7 +416,7 @@ g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end)
  *
  * Gets whether or not @stream owns the backend file descriptor.
  *
- * Returns %TRUE if @stream owns the backend file descriptor or %FALSE
+ * Returns: %TRUE if @stream owns the backend file descriptor or %FALSE
  * otherwise.
  **/
 gboolean
